/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.service;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;

import java.lang.reflect.Type;
import java.util.Map;

import org.oransc.rappmanager.client.AsyncRestClient;
import org.oransc.rappmanager.client.AsyncRestClientFactory;
import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.controller.Chart;
import org.oransc.rappmanager.exception.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.MultipartBodyBuilder;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.BodyInserters.MultipartInserter;
import reactor.core.publisher.Mono;

// Need to support multiple Chart registry. So add an interface and use it based on the registry choice.
@Service
public class ChartService {

    private static final Logger logger = LoggerFactory.getLogger(ChartService.class);

    private final AsyncRestClient chartClient;

    private static Gson gson = new GsonBuilder().serializeNulls().create();

    private static String basePath = "/api/charts";

    public ChartService(@Autowired ApplicationConfig appConfig) {
        AsyncRestClientFactory factory = new AsyncRestClientFactory(appConfig.getWebClientConfig());
        chartClient = factory.createRestClientNoHttpProxy(appConfig.getChartApi());
    }

    public Map<String, Chart[]> getCharts() throws ServiceException {
        ResponseEntity<String> rsp = chartClient.getForEntity(basePath).block();
        if (rsp.getStatusCodeValue() != 200) {
            throw new ServiceException("Error Getting Charts");
        }
        logger.debug("Sucessfully fetched the charts");
        return parseChart(rsp.getBody());
    }

    public Chart[] getChart(String name) throws ServiceException {
        ResponseEntity<String> rsp = chartClient.getForEntity(basePath + "/" + name).block();
        if (rsp.getStatusCodeValue() != 200) {
            throw new ServiceException("Error Getting Chart " + name);
        }
        logger.debug("Sucessfully fetched the chart");
        return gson.fromJson(rsp.getBody(), Chart[].class);
    }

    private MultipartInserter createPostFileBody(MultipartFile chartFile) {
        MultipartBodyBuilder bodyBuilder = new MultipartBodyBuilder();
        bodyBuilder.part("file", chartFile.getResource());
        return BodyInserters.fromMultipartData(bodyBuilder.build());
    }

    // Not working. Need to check
    public Mono<ResponseEntity<String>> saveChart(Chart chart, MultipartFile chartFile) {
        MultipartInserter body = createPostFileBody(chartFile);
        return Mono.just(new ResponseEntity<>(chart.getName(), HttpStatus.OK)); // Dummy return for now
        // return chartClient.postForEntity(basePath, body,
        // MediaType.MULTIPART_FORM_DATA);
    }

    public void deleteChart(String name, String version) throws ServiceException {
        ResponseEntity<String> rsp = chartClient.deleteForEntity(basePath + "/" + name + "/" + version).block();
        if (rsp.getStatusCodeValue() != 200) {
            throw new ServiceException("Error in deleting Chart " + name);
        }
        logger.debug("Sucessfully deleted the chart");
    }

    private Map<String, Chart[]> parseChart(String json) {
        Type mapType = new TypeToken<Map<String, Chart[]>>() {}.getType();
        return gson.fromJson(json, mapType);
    }

}
